==> google-cloud-sdk-357.0.0.tgz: google-cloud-sdk/lib/googlecloudsdk/command_lib/storage/tasks/cp/file_upload_task.py <==
# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Task for file uploads.

Typically executed in a task iterator:
googlecloudsdk.command_lib.storage.tasks.task_executor.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import math
import os
import random

from googlecloudsdk.api_lib.storage import api_factory
from googlecloudsdk.api_lib.storage import cloud_api
from googlecloudsdk.api_lib.storage import gcs_api
from googlecloudsdk.command_lib.storage import tracker_file_util
from googlecloudsdk.command_lib.storage.tasks import task
from googlecloudsdk.command_lib.storage.tasks import task_executor
from googlecloudsdk.command_lib.storage.tasks.cp import copy_component_util
from googlecloudsdk.command_lib.storage.tasks.cp import file_part_upload_task
from googlecloudsdk.command_lib.storage.tasks.cp import finalize_composite_upload_task
from googlecloudsdk.core import properties
from googlecloudsdk.core.util import scaled_integer


def _get_component_count(file_size, api_max_component_count):
  """Returns the number of components to use for an upload."""
  preferred_component_size = scaled_integer.ParseInteger(
      properties.VALUES.storage.parallel_composite_upload_component_size.Get())
  component_count = math.ceil(file_size / preferred_component_size)

  if component_count < 2:
    return 2
  if component_count > api_max_component_count:
    return api_max_component_count
  return component_count


def _get_random_prefix():
  """Returns an ID distinguishing upload components from different machines."""
  return str(random.randint(1, 10**10))


class FileUploadTask(task.Task):
  """Represents a command operation triggering a file upload."""

  def __init__(self,
               source_resource,
               destination_resource,
               user_request_args=None):
    """Initializes task.

    Args:
      source_resource (resource_reference.FileObjectResource): Must contain
          local filesystem path to upload object. Does not need to contain
          metadata.
      destination_resource (resource_reference.ObjectResource|UnknownResource):
          Must contain the full object path. Directories will not be accepted.
          Existing objects at the this location will be overwritten.
      user_request_args (UserRequestArgs|None): Values for RequestConfig.
    """
    super(FileUploadTask, self).__init__()
    self._source_resource = source_resource
    self._destination_resource = destination_resource
    self._user_request_args = user_request_args
    self.parallel_processing_key = (
        self._destination_resource.storage_url.url_string)

    self._composite_upload_threshold = scaled_integer.ParseInteger(
        properties.VALUES.storage.parallel_composite_upload_threshold.Get())

  def execute(self, task_status_queue=None):
    source_filename = self._source_resource.storage_url.object_name
    size = os.path.getsize(source_filename)

    destination_provider = self._destination_resource.storage_url.scheme
    api_capabilties = api_factory.get_capabilities(destination_provider)
    should_perform_single_transfer = (
        size < self._composite_upload_threshold or
        not self._composite_upload_threshold or
        cloud_api.Capability.COMPOSE_OBJECTS not in api_capabilties or
        not task_executor.should_use_parallelism()
    )

    if should_perform_single_transfer:
      file_part_upload_task.FilePartUploadTask(
          self._source_resource,
          self._destination_resource,
          offset=0,
          length=size,
          user_request_args=self._user_request_args).execute(task_status_queue)
    else:
      component_size_property = (
          properties.VALUES.storage.parallel_composite_upload_component_size)
      component_offsets_and_lengths = (
          copy_component_util.get_component_offsets_and_lengths(
              size,
              component_size_property.Get(),
              gcs_api.MAX_OBJECTS_PER_COMPOSE_CALL))

      tracker_file_path = tracker_file_util.get_tracker_file_path(
          self._destination_resource.storage_url,
          tracker_file_util.TrackerFileType.PARALLEL_UPLOAD,
          source_url=self._source_resource.storage_url)
      tracker_data = tracker_file_util.read_composite_upload_tracker_file(
          tracker_file_path)

      if tracker_data:
        random_prefix = tracker_data.random_prefix
      else:
        random_prefix = _get_random_prefix()

      tracker_file_util.write_composite_upload_tracker_file(
          tracker_file_path, random_prefix)

      file_part_upload_tasks = []
      for i, (offset, length) in enumerate(component_offsets_and_lengths):

        temporary_component_resource = (
            copy_component_util.get_temporary_component_resource(
                self._source_resource, self._destination_resource,
                random_prefix, i))

        upload_task = file_part_upload_task.FilePartUploadTask(
            self._source_resource,
            temporary_component_resource,
            offset,
            length,
            component_number=i,
            total_components=len(component_offsets_and_lengths),
            user_request_args=self._user_request_args)

        file_part_upload_tasks.append(upload_task)

      finalize_upload_task = (
          finalize_composite_upload_task.FinalizeCompositeUploadTask(
              expected_component_count=len(file_part_upload_tasks),
              source_resource=self._source_resource,
              destination_resource=self._destination_resource,
              random_prefix=random_prefix))

      return task.Output(
          additional_task_iterators=[
              file_part_upload_tasks,
              [finalize_upload_task]
          ],
          messages=None)

==> google-cloud-sdk-379.0.0.tgz: google-cloud-sdk/lib/googlecloudsdk/command_lib/storage/tasks/cp/file_upload_task.py <==
# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Task for file uploads.

Typically executed in a task iterator:
googlecloudsdk.command_lib.storage.tasks.task_executor.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import math
import os
import random

from googlecloudsdk.api_lib.storage import api_factory
from googlecloudsdk.api_lib.storage import cloud_api
from googlecloudsdk.api_lib.storage import gcs_api
from googlecloudsdk.command_lib.storage import tracker_file_util
from googlecloudsdk.command_lib.storage.tasks import task
from googlecloudsdk.command_lib.storage.tasks import task_util
from googlecloudsdk.command_lib.storage.tasks.cp import copy_component_util
from googlecloudsdk.command_lib.storage.tasks.cp import copy_util
from googlecloudsdk.command_lib.storage.tasks.cp import file_part_upload_task
from googlecloudsdk.command_lib.storage.tasks.cp import finalize_composite_upload_task
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core.util import scaled_integer


def _get_component_count(file_size, api_max_component_count):
  """Returns the number of components to use for an upload."""
  preferred_component_size = scaled_integer.ParseInteger(
      properties.VALUES.storage.parallel_composite_upload_component_size.Get())
  component_count = math.ceil(file_size / preferred_component_size)

  if component_count < 2:
    return 2
  if component_count > api_max_component_count:
    return api_max_component_count
  return component_count


def _get_random_prefix():
  """Returns an ID distinguishing upload components from different machines."""
  return str(random.randint(1, 10**10))


class FileUploadTask(task.Task):
  """Represents a command operation triggering a file upload."""

  def __init__(self,
               source_resource,
               destination_resource,
               delete_source=False,
               print_created_message=False,
               user_request_args=None):
    """Initializes task.

    Args:
      source_resource (resource_reference.FileObjectResource): Must contain
        local filesystem path to upload object. Does not need to contain
        metadata.
      destination_resource (resource_reference.ObjectResource|UnknownResource):
        Must contain the full object path. Directories will not be accepted.
        Existing objects at the this location will be overwritten.
      delete_source (bool): If copy completes successfully, delete the source
        object afterwards.
      print_created_message (bool): Print a message containing the versioned
        URL of the copy result.
      user_request_args (UserRequestArgs|None): Values for RequestConfig.
    """
    super(FileUploadTask, self).__init__()
    self._source_resource = source_resource
    self._destination_resource = destination_resource
    self._delete_source = delete_source
    self._print_created_message = print_created_message
    self._user_request_args = user_request_args

    self.parallel_processing_key = (
        self._destination_resource.storage_url.url_string)

    self._composite_upload_threshold = scaled_integer.ParseInteger(
        properties.VALUES.storage.parallel_composite_upload_threshold.Get())

  def execute(self, task_status_queue=None):
    destination_provider = self._destination_resource.storage_url.scheme
    if copy_util.check_for_cloud_clobber(
        self._user_request_args, api_factory.get_api(destination_provider),
        self._destination_resource):
      log.status.Print(
          copy_util.get_no_clobber_message(
              self._destination_resource.storage_url))
      return

    source_url = self._source_resource.storage_url
    source_filename = source_url.object_name

    if source_url.is_pipe:
      size = None
    else:
      size = os.path.getsize(source_filename)

    api_capabilties = api_factory.get_capabilities(destination_provider)
    should_perform_single_transfer = (
        source_url.is_pipe or size < self._composite_upload_threshold or
        not self._composite_upload_threshold or
        cloud_api.Capability.COMPOSE_OBJECTS not in api_capabilties or
        not task_util.should_use_parallelism())

    if should_perform_single_transfer:
      task_output = file_part_upload_task.FilePartUploadTask(
          self._source_resource,
          self._destination_resource,
          offset=0,
          length=size,
          user_request_args=self._user_request_args).execute(task_status_queue)
      if self._print_created_message:
        for message in task_output.messages:
          if message.topic == task.Topic.UPLOADED_COMPONENT:
            log.status.Print('Created: {}'.format(
                message.payload.object_resource.storage_url))
            break
      if self._delete_source:
        os.remove(self._source_resource.storage_url.object_name)
    else:
      component_size_property = (
          properties.VALUES.storage.parallel_composite_upload_component_size)
      component_offsets_and_lengths = (
          copy_component_util.get_component_offsets_and_lengths(
              size,
              component_size_property.Get(),
              gcs_api.MAX_OBJECTS_PER_COMPOSE_CALL))

      tracker_file_path = tracker_file_util.get_tracker_file_path(
          self._destination_resource.storage_url,
          tracker_file_util.TrackerFileType.PARALLEL_UPLOAD,
          source_url=source_url)
      tracker_data = tracker_file_util.read_composite_upload_tracker_file(
          tracker_file_path)

      if tracker_data:
        random_prefix = tracker_data.random_prefix
      else:
        random_prefix = _get_random_prefix()

      tracker_file_util.write_composite_upload_tracker_file(
          tracker_file_path, random_prefix)

      file_part_upload_tasks = []
      for i, (offset, length) in enumerate(component_offsets_and_lengths):

        temporary_component_resource = (
            copy_component_util.get_temporary_component_resource(
                self._source_resource, self._destination_resource,
                random_prefix, i))

        upload_task = file_part_upload_task.FilePartUploadTask(
            self._source_resource,
            temporary_component_resource,
            offset,
            length,
            component_number=i,
            total_components=len(component_offsets_and_lengths),
            user_request_args=self._user_request_args)

        file_part_upload_tasks.append(upload_task)

      finalize_upload_task = (
          finalize_composite_upload_task.FinalizeCompositeUploadTask(
              expected_component_count=len(file_part_upload_tasks),
              source_resource=self._source_resource,
              destination_resource=self._destination_resource,
              random_prefix=random_prefix,
              delete_source=self._delete_source,
              print_created_message=self._print_created_message))

      return task.Output(
          additional_task_iterators=[
              file_part_upload_tasks,
              [finalize_upload_task]
          ],
          messages=None)
